import * as ed25519 from '@noble/ed25519';
import { sha3_256 } from '@noble/hashes/sha3';
import { utf8, base32, base64 } from '@scure/base';
import { concatBytes } from 'micro-packed';
const ADDRESS_VERSION = new Uint8Array([0x03]);
export function formatPublicKey(pubBytes) {
    const checksum = sha3_256(concatBytes(utf8.decode('.onion checksum'), pubBytes, ADDRESS_VERSION));
    const addr = concatBytes(pubBytes, checksum.slice(0, 2), ADDRESS_VERSION);
    return `${base32.encode(addr).toLowerCase()}.onion`;
}
export function parseAddress(address) {
    if (!address.endsWith('.onion'))
        throw new Error('Address must end with .onion');
    const addr = base32.decode(address.replace(/\.onion$/, '').toUpperCase());
    const skip = addr.slice(0, addr.length - 3);
    const key = formatPublicKey(skip);
    if (key !== address)
        throw new Error('Invalid checksum');
    return skip;
}
export async function getKeys(seed) {
    const { head, prefix, pointBytes } = await ed25519.utils.getExtendedPublicKey(seed);
    const added = concatBytes(head, prefix);
    return {
        publicKeyBytes: pointBytes,
        publicKey: formatPublicKey(pointBytes),
        privateKey: `ED25519-V3:${base64.encode(added)}`,
    };
}
export default getKeys;
